@@ -22,6 +22,8 @@ Contributors:
 #include "config.h"
 
 #include <ctype.h>
+#include <errno.h>
+#include <limits.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -32,8 +34,13 @@ Contributors:
 #  include <aclapi.h>
 #  include <io.h>
 #  include <lmcons.h>
+#  include <fcntl.h>
+#  define PATH_MAX MAX_PATH
 #else
 #  include <sys/stat.h>
+#  include <pwd.h>
+#  include <grp.h>
+#  include <unistd.h>
 #endif
 
 #include "misc_mosq.h"
@@ -45,6 +52,8 @@ FILE *mosquitto__fopen(const char *path, const char *mode, bool restrict_read)
 #ifdef WIN32
 	char buf[4096];
 	int rc;
+	int flags = 0;
+
 	rc = ExpandEnvironmentStringsA(path, buf, 4096);
 	if(rc == 0 || rc > 4096){
 		return NULL;
@@ -64,9 +73,11 @@ FILE *mosquitto__fopen(const char *path, const char *mode, bool restrict_read)
 			switch(mode[0]){
 				case 'a':
 					dwCreationDisposition = OPEN_ALWAYS;
+					flags = _O_APPEND;
 					break;
 				case 'r':
 					dwCreationDisposition = OPEN_EXISTING;
+					flags = _O_RDONLY;
 					break;
 				case 'w':
 					dwCreationDisposition = CREATE_ALWAYS;
@@ -88,6 +99,7 @@ FILE *mosquitto__fopen(const char *path, const char *mode, bool restrict_read)
 				return NULL;
 			}
 
+			memset(&sec, 0, sizeof(sec));
 			sec.nLength = sizeof(SECURITY_ATTRIBUTES);
 			sec.bInheritHandle = FALSE;
 			sec.lpSecurityDescriptor = &sd;
@@ -100,7 +112,7 @@ FILE *mosquitto__fopen(const char *path, const char *mode, bool restrict_read)
 
 			LocalFree(pacl);
 
-			fd = _open_osfhandle((intptr_t)hfile, 0);
+			fd = _open_osfhandle((intptr_t)hfile, flags);
 			if (fd < 0) {
 				return NULL;
 			}
@@ -110,6 +122,9 @@ FILE *mosquitto__fopen(const char *path, const char *mode, bool restrict_read)
 				_close(fd);
 				return NULL;
 			}
+			if(mode[0] == 'a'){
+				fseek(fptr, 0, SEEK_END);
+			}
 			return fptr;
 
 		}else {
@@ -117,30 +132,87 @@ FILE *mosquitto__fopen(const char *path, const char *mode, bool restrict_read)
 		}
 	}
 #else
-	if(mode[0] == 'r'){
-		struct stat statbuf;
-		if(stat(path, &statbuf) < 0){
-			return NULL;
-		}
-
-		if(!S_ISREG(statbuf.st_mode) && !S_ISLNK(statbuf.st_mode)){
-			log__printf(NULL, MOSQ_LOG_ERR, "Error: %s is not a file.", path);
-			return NULL;
-		}
-	}
+	FILE *fptr;
+	struct stat statbuf;
 
 	if (restrict_read) {
-		FILE *fptr;
 		mode_t old_mask;
 
 		old_mask = umask(0077);
 		fptr = fopen(path, mode);
 		umask(old_mask);
-
-		return fptr;
 	}else{
-		return fopen(path, mode);
+		fptr = fopen(path, mode);
 	}
+	if(!fptr) return NULL;
+
+	if(fstat(fileno(fptr), &statbuf) < 0){
+		fclose(fptr);
+		return NULL;
+	}
+
+	if(restrict_read){
+		if(statbuf.st_mode & S_IRWXO){
+#ifdef WITH_BROKER
+			log__printf(NULL, MOSQ_LOG_WARNING,
+#else
+			fprintf(stderr,
+#endif
+					"Warning: File %s has world readable permissions. Future versions will refuse to load this file.",
+					path);
+#if 0
+			return NULL;
+#endif
+		}
+		if(statbuf.st_uid != getuid()){
+			char buf[4096];
+			struct passwd pw, *result;
+
+			getpwuid_r(getuid(), &pw, buf, sizeof(buf), &result);
+			if(result){
+#ifdef WITH_BROKER
+				log__printf(NULL, MOSQ_LOG_WARNING,
+#else
+				fprintf(stderr,
+#endif
+						"Warning: File %s owner is not %s. Future versions will refuse to load this file.",
+						path, result->pw_name);
+			}
+#if 0
+			// Future version
+			return NULL;
+#endif
+		}
+		if(statbuf.st_gid != getgid()){
+			char buf[4096];
+			struct group grp, *result;
+
+			getgrgid_r(getgid(), &grp, buf, sizeof(buf), &result);
+			if(result){
+#ifdef WITH_BROKER
+				log__printf(NULL, MOSQ_LOG_WARNING,
+#else
+				fprintf(stderr,
+#endif
+						"Warning: File %s group is not %s. Future versions will refuse to load this file.",
+						path, result->gr_name);
+			}
+#if 0
+			// Future version
+			return NULL
+#endif
+		}
+	}
+
+
+	if(!S_ISREG(statbuf.st_mode) && !S_ISLNK(statbuf.st_mode)){
+#ifdef WITH_BROKER
+		log__printf(NULL, MOSQ_LOG_ERR, "Error: %s is not a file.", path);
+#endif
+		fclose(fptr);
+		return NULL;
+	}
+	return fptr;
 #endif
 }
 
